#include "cmd_panel_imp.h"
#include "drawset.h"
#include "mainframe.h"
#include "selection.h"
#include "mlsmenu.h"
#include "archive.h"

using namespace MLS;
using namespace MLSUTIL;

void CmdPanelImp::UpdateConfig()
{
}

void CmdPanelImp::SaveConfig()
{
}

///	@brief	명령어를 실행한다.
///	@param	cmd		명령어및 인자
///	@param	bPause	명령어를 실행시키고 종료시 잠시 정지시킬지 여부
/// @param	bFork	Fork할 것인지?
///	@return	명령어 실행 결과
int CmdPanelImp::Run( const std::string &cmd, bool bPause, bool bBackground)
{
	// command 창에서 exit, quit 를 치면 종료하게 한다.
	if (cmd == "exit" || cmd == "quit") 
	{
		Quit();
		return 0;
	}

	CursesDestroy();
	int status;

	if (bBackground)
	{
		cout << "mls $ " << cmd.c_str() << endl;
		string sCmd = cmd + " > /dev/null 2>&1 &";
		status = system(sCmd.c_str());
	}
	else
	{
		cout << "mls $ " << cmd.c_str() << endl;
		status = system(cmd.c_str());
	}

	if (bPause)
	{
		cbreak();
		noecho();

		cout << endl << gettext("Press any key to return Mls...") << '\r' << endl;
		getch();
	}

	CursesInit();
	return 0;
}

///	@brief	모든 커맨드 메시지를 관리
///	@param	p	실행 시킬 명령어
///	@return		명령어 실행 결과
int CmdPanelImp::ParseAndRun(const string &p, bool Pause)
{
	bool	bFlag  = false,
		 	bPause = Pause,
		 	bParam = false,
		 	bMcd   = false ,
		 	bConfirm = false,
			bBackground = false,
			bRootExe = false;
			
	// cd 구현
	if (p.substr(0, 2) == "cd")
	{
		string sStr;
		if (p == "cd" || p == "cd ")
			sStr = _pPanel->GetReader()->GetRealPath("~");
		else
			sStr = _pPanel->GetReader()->GetRealPath(p.substr(3));
		sStr = Replace(sStr, "\\", "");
		if (sStr != "")	
		{
			if (_pPanel->Read(sStr) == true) 
			{
				if (g_tMainFrame.GetScreenSync())
					ViewSync();
				return 1;
			}
		}
		return 0;
	}
	
	string 	arg, tmp_name;

	File	tFile;
	
	// 파일을 복사 해서 보여주기 위한...
	{
		File* 	pFile = _pPanel->GetCurFile();
		if (_pPanel->GetReader()->View(pFile, &tFile) == false)
		{
			//MsgBox(	gettext("Error"), gettext("file reading failure !!!!"));
			return 0;
		}
	}
	
	// 파싱
	for (string::const_iterator i = p.begin(); i!= p.end(); i++)
	{
		if (*i == '%')
		{
			bFlag = true;
			continue;
		}
		
		if (bFlag)
		{
			switch(*i)
			{
			// 파일을 리턴해줌
			case '1':
			case 'F':
				arg += addslash(tFile.sFullName);
				break;
				
			case 'N':
			{
				if (tFile.sName[0]=='.') return 0;
				
				string::size_type p = tFile.sName.rfind('.');
				
				if (p != string::npos)
					arg += addslash(tFile.sFullName.substr(0, p-1));
				else
					arg += addslash(tFile.sFullName);
				break;
			}
			
			case 'E':
				if (tFile.Ext().empty()) return 0;
				arg += addslash(tFile.Ext());
				break;
				
			case 'W':
				bPause = true;
				break;
				
			case '%':
				arg += '%';
				break;
				
			case 'S':
			{
				Selection tSelection;
				_pPanel->GetSelection(tSelection);
				
				if (tSelection.GetSize() == 0) return 0;				
				bool first = true;				
			
				for (int t = 0; t< tSelection.GetSize(); t++)
				{
					if (first) first= false;
					else arg += ' ';
					arg += addslash(tSelection[t]->sFullName);
				}
				break;
			}
			
			case 's':
			{
				Selection tSelection;
				_pPanel->GetSelection(tSelection);
				
				if (!tSelection.GetSize() == 0)
				{
					bool first = true;					
				
					for (int t = 0; t< tSelection.GetSize(); t++)
					{
						if (first) first= false;
						else arg += ' ';
					
						arg += addslash(tSelection[t]->sFullName);
					}
				}
				else 
				{
					if (tFile.sName == "..") return 0;
					arg += addslash(tFile.sFullName);
				}
				break;
			}
			
			case 'd':			
				arg += addslash(_pPanel->GetPath());
				break;		
			
			case 'D':
				bMcd = true;
				break;
			
			case 'P':
				bParam= true;
				break;

			case 'R': // root 계정 실행시
				bRootExe = true;
				break;
				
			case 'H':
				arg += addslash(_pPanel->GetReader()->GetRealPath("~"));
				break;
				
			case 'Q':
				bConfirm = true;
				break;
				
			case 'B':
				bBackground = true;
				break;
				
			default:
				break;
			}
			
			bFlag = false;
			continue;
		}
		else arg += *i;
	}
	
	if (bMcd)
	{
		/*
		Mcd mcd(_tPanel);
		mcd.setCur(_tPanel->GetCurrentPath());
		if (!mcd.proc())
            return 0;
		arg += addslash(mcd.getCur());
		*/
	}
	
	if (bParam)	if (InputBox(gettext("Edit Parameter"), arg)<0) return 0;
		
	if (bConfirm)
		if (YNBox(gettext("Do you want to run this operation?"), false) != true) return 0;

	if (bRootExe)
		arg = "su - --command=\"" + arg + "\"";
	
	// 실행
	int nRt = Run(arg, bPause, bBackground);
	return 0;
}

void	CmdPanelImp::SplitViewSync()
{
	g_tMainFrame.SetScreenSync(!g_tMainFrame.GetScreenSync());

	if (g_tMainFrame.GetScreenSync())
	{
		NCurses_Panel* pPanel = g_tMainFrame.GetPanel(1);
		NCurses_Mcd* pMcd = g_tMainFrame.GetMcd(0);
		
		void*	pWait = MsgWaitBox(	gettext("Wait"),
									gettext("Please wait !!! - Cancel Key [ESC]"));
		pMcd->Destroy();
		pMcd->SetReader(pPanel->GetReader());
		pMcd->Rescan(2);
		pMcd->AddDirectory(pPanel->GetReader()->GetPath());
		pMcd->SetCur(pPanel->GetReader()->GetPath());
		MsgWaitEnd(pWait);
		ViewSync();
	}
}

void	CmdPanelImp::ViewSync()
{
	int nActive = g_tMainFrame.GetActiveNum();

	NCurses_Panel* pPanel = g_tMainFrame.GetPanel(1);
	NCurses_Mcd* pMcd = g_tMainFrame.GetMcd(0);
	
	pMcd->AddDirectory(pPanel->GetReader()->GetPath());
	pMcd->SetCur(pPanel->GetReader()->GetPath());
	pPanel->_bFocus = true;
	pMcd->_bFocus = true;
	g_tMainFrame.Refresh(false);
}

void CmdPanelImp::Key_Enter()
{
	if (g_tMainFrame.GetScreenSync())
	{
		NCurses_Panel* pPanel = g_tMainFrame.GetPanel(1);
		NCurses_Mcd* pMcd = g_tMainFrame.GetMcd(0);
		
		Reader*  pBefReader = pPanel->GetReader();
		pPanel->Key_Enter();
		
		if (pPanel->GetReader() != pBefReader)
		{
			pMcd->Destroy();
			pMcd->SetReader(pPanel->GetReader());
			pMcd->Rescan(2);
			pMcd->AddDirectory(pPanel->GetReader()->GetPath());
			pMcd->SetCur(pPanel->GetReader()->GetPath());
			pMcd->_bFocus = true;
		}
		else
		{
			pMcd->AddDirectory(pPanel->GetReader()->GetPath());
			pMcd->SetCur(pPanel->GetReader()->GetPath());
		}
		
		g_tMainFrame.Refresh(false);
	}
	else
	{
		_pPanel->Key_Enter();
	}
}

void CmdPanelImp::Key_Left()
{ 
	_pPanel->Key_Left();		
	if (g_tMainFrame.GetScreenSync()) ViewSync();
}

void CmdPanelImp::Key_Right() 		
{ 
	_pPanel->Key_Right();
	if (g_tMainFrame.GetScreenSync()) ViewSync();
}

void CmdPanelImp::Key_Up()
{ 
	_pPanel->Key_Up();
	if (g_tMainFrame.GetScreenSync()) ViewSync();
}

void CmdPanelImp::Key_Down()
{ 
	_pPanel->Key_Down();
	if (g_tMainFrame.GetScreenSync()) ViewSync();
}

void CmdPanelImp::Key_Home()
{ 
	_pPanel->Key_Home();
	if (g_tMainFrame.GetScreenSync()) ViewSync();
}

void CmdPanelImp::Key_End() 		
{ 
	_pPanel->Key_End();
	if (g_tMainFrame.GetScreenSync()) ViewSync();
}

void CmdPanelImp::Key_PageUp()
{
	_pPanel->Key_PageUp();
	if (g_tMainFrame.GetScreenSync()) ViewSync();
}

void CmdPanelImp::Key_PageDown()
{
	_pPanel->Key_PageDown();
	if (g_tMainFrame.GetScreenSync()) ViewSync();
}

void	CmdPanelImp::Mcd()
{
	g_tMainFrame.SetScreenSync(false);

	void*	pWait = MsgWaitBox(	gettext("Wait"),
								gettext("Please wait !!! - Cancel Key [ESC]"));
	_pMcd->Destroy();
	_pMcd->SetReader(_pPanel->GetReader());
	_pMcd->Rescan(2);
	_pMcd->AddDirectory(_pPanel->GetReader()->GetPath());
	_pMcd->SetCur(_pPanel->GetReader()->GetPath());
	MsgWaitEnd(pWait);
	g_tMainFrame.SetActiveViewType(MCD);
	_pMcd->_bFocus = true;
}

void CmdPanelImp::View()
{
	File*	pFile = _pPanel->GetCurFile();

	if (pFile == NULL || pFile->bDir == true) 
	{
		MsgBox(gettext("Error"), gettext("Don't directory view."));
		return;
	}
		
	if (g_tMainFrame.GetActiveViewType() != PANEL) return;

	if (g_tCfg.GetValue("Default", "ExtViewer").empty())
	{
		EditorChoice(false);
	}
	else
	{
		EditorChoice(false, g_tCfg.GetValue("Default", "ExtViewer"));
	}
}

void CmdPanelImp::Editor()
{
	File*	pFile = _pPanel->GetCurFile();

	if (pFile == NULL || pFile->bDir == true) 
	{
		MsgBox(gettext("Error"), gettext("Don't directory edit."));
		return;
	}

	if (g_tMainFrame.GetActiveViewType() != PANEL) return;

	if (g_tCfg.GetValue("Default", "ExtEditor").empty())
	{
		EditorChoice(false);
	}
	else
	{
		EditorChoice(false, g_tCfg.GetValue("Default", "ExtEditor"));
	}
}

void CmdPanelImp::ExtEditor()
{
	if (g_tMainFrame.GetActiveViewType() != PANEL) return;

	if (g_tCfg.GetValue("Default", "ExtEditor").empty())
		EditorChoice(false, "vi");
	else
		EditorChoice(false, g_tCfg.GetValue("Default", "ExtEditor"));
}

void CmdPanelImp::EditorChoice(bool bReadOnly, const string& sEditorCmd, File* pFile)
{
	if ( pFile == NULL )
	{
		pFile = _pPanel->GetCurFile();
		if (!pFile) return;
	}

	File 	tFile;
	if (_pPanel->GetReader()->View(pFile, &tFile) == false) return;

	bool bExtEditor = false;
	if (!sEditorCmd.empty()) bExtEditor = true;

	if (!bExtEditor)
	{
		g_tMainFrame.SetScreenSync(false);

		void*	pWait = MsgWaitBox(	gettext("Wait"),
									gettext("Please wait !!! - Cancel Key [ESC]"));
		
		int             nSize = 0;
		string  sSize = g_tCfg.GetValue("Default", "EditorTabSize");
		if (sSize.size() == 0)	nSize = 8;
		else					nSize = atoi(sSize.c_str());

		bool bEditorLineNum = g_tCfg.GetBool("Default", "EditorLineNum");
		bool bEditorBackup = g_tCfg.GetBool("Default", "EditorBackup");

		const string&	sReaderName = _pPanel->GetReader()->GetReaderName();

		if (bReadOnly == false) 
			bReadOnly = (sReaderName == "archive") ? true : false;

		// title 은 원본을 보여준다.
		string	sTitle = sReaderName + ":/" + isKorCode(pFile->sFullName);

		_pEditor->SetViewTitle( sTitle );
		_pEditor->SetEditor(nSize, bEditorBackup, bEditorLineNum);
		
		if (_pEditor->Load(tFile.sFullName.c_str(), bReadOnly) == false)
		{
			MsgWaitEnd(pWait);
			MsgBox( gettext("Error"), gettext("File loading failure."));
		}
		else
		{
			g_tMainFrame.SetActiveViewType(EDITOR);
			MsgWaitEnd(pWait);
			_pEditor->_bFocus = true;
			Refresh();
		}
	}
	else
	{
		g_Log.Write("ParseAndRun !!! [%s]", sEditorCmd.c_str());
	
		if (sEditorCmd.empty())
			ParseAndRun("vi " + tFile.sFullName);
		else
			ParseAndRun(sEditorCmd + " " + tFile.sFullName);
	}
	return;
}

void CmdPanelImp::Refresh()
{
	g_tMainFrame.GetPanel(0)->Read(g_tMainFrame.GetPanel(0)->GetPath());
	g_tMainFrame.GetPanel(1)->Read(g_tMainFrame.GetPanel(1)->GetPath());

	g_tMainFrame.Refresh();
}

void CmdPanelImp::Menu()
{
	MlsMenu		tMlsMenu;
	vector<string>	vDisableList;

	const string&	sReaderName = _pPanel->GetReader()->GetReaderName();

	if (sReaderName == "file")
	{
		vDisableList.push_back("Cmd_RemoteClose");
	}
	else if (sReaderName == "ftp" || sReaderName == "sftp")
	{
		vDisableList.push_back("Cmd_RemoteConnect");
	}
	else
	{
		vDisableList.push_back("Cmd_RemoteConnect");
		vDisableList.push_back("Cmd_RemoteClose");
	}

	tMlsMenu.SetDisable(vDisableList);
	tMlsMenu.Create();
	tMlsMenu.Do();

	g_tMainFrame.Refresh(false);
	if (tMlsMenu.GetCurCmd().size() != 0)
	{
		g_Log.Write("Menu Run [%s]", tMlsMenu.GetCurCmd().c_str());
		g_tMainFrame.GetCommand()->Execute(tMlsMenu.GetCurCmd());
	}
}

void CmdPanelImp::Quit()
{
	if (g_tCfg.GetBool("Default", "AskMlsExit") == true)
	{
		bool bYN = YNBox(gettext("Mls Quit"), gettext("Do you really want to quit the mls ?"), true);
		if (bYN == false) return;
	}
	
	// 마지막 mls 가 마지막으로 사용한 디렉토리를 저장한다.
	if (_pPanel->GetReader()->GetInitType() == "file://")
	{
		FILE *fp= fopen((g_tCfg.GetValue("CfgHome") + "path").c_str(), "wb");

		if (fp)
		{
			fputs(_pPanel->GetPath().c_str(), fp);
			fclose(fp);
		}
	}
	g_tMainFrame.Exit();
}

void CmdPanelImp::Split()
{
	g_tMainFrame.Split();
}

void CmdPanelImp::NextWindow()
{
	g_tMainFrame.NextWindow();
}

void CmdPanelImp::ArchiveFileView()
{
	if (g_tMainFrame.GetActiveViewType() != PANEL) return;

	File* pFile = _pPanel->GetCurFile();

	g_Log.Write("ArchiveFileView [%s]", pFile->sFullName.c_str());
	if (pFile->sType.substr(0, 7) == "archive")
		MsgBox(gettext("Error"), "Archive file view failure !!!");
	else
	{
		if (_pPanel->GetReader()->GetReaderName() != "file")
			_pPanel->PluginOpen("/tmp/" + pFile->sName, "archive");
		else
			_pPanel->PluginOpen(pFile->sFullName, "archive");
	}
}

void CmdPanelImp::ArchiveFileMake()
{
	if (g_tMainFrame.GetActiveViewType() != PANEL) return;	
}

void CmdPanelImp::RemoteConnect()
{
	if (g_tMainFrame.GetActiveViewType() != PANEL) return;

	string sConnectionInfo;
	if (InputBox(	gettext("Input sftp(ftp) connect url (sftp://user:pswd@hostname)"),
							sConnectionInfo) == ERROR) return;

	if (sConnectionInfo.size() > 6)
	{
		if (sConnectionInfo.substr(0, 6) == "ftp://")
		{
			sConnectionInfo = sConnectionInfo.substr(6);
			_pPanel->PluginOpen(sConnectionInfo, "ftp");
		}
		else if (sConnectionInfo.substr(0, 7) == "sftp://")
		{
			sConnectionInfo = sConnectionInfo.substr(7);
			_pPanel->PluginOpen(sConnectionInfo, "sftp");
		}
		else
		{
			_pPanel->PluginOpen(sConnectionInfo, "sftp");
		}
	}
	else
		MsgBox(gettext("Error"), gettext("input url invalid format."));
	g_tMainFrame.Refresh();
}

void CmdPanelImp::RemoteClose()
{
	if (g_tMainFrame.GetActiveViewType() != PANEL) return;

	_pPanel->PluginClose("file");
	Refresh();
}

void CmdPanelImp::Remove()
{
	Selection& tMainSelection = *(g_tMainFrame.GetSelection());
	tMainSelection.Clear();
	_pPanel->GetSelection(tMainSelection);
	tMainSelection.ExpandDir(_pPanel->GetReader(), false);
	_pPanel->GetReader()->Remove(tMainSelection);
	Refresh();
}

void CmdPanelImp::CopyClip()
{
	Selection& tMainSelection = *(g_tMainFrame.GetSelection());
	tMainSelection.Clear();
	_pPanel->GetSelection(tMainSelection);
	tMainSelection.SetSelectPath(_pPanel->GetReader()->GetPath());
	tMainSelection.SetReader(_pPanel->GetReader());
	
	if (tMainSelection.GetSize() == 0)
		g_tMainFrame.GetClip()->Set(CLIP_NONE);
	else
		g_tMainFrame.GetClip()->Set(CLIP_COPY);

	g_Log.Write("CopyClip [%s]", _pPanel->GetReader()->GetPath().c_str());
}

void CmdPanelImp::CutClip()
{
	Selection& tMainSelection = *(g_tMainFrame.GetSelection());
	tMainSelection.Clear();
	_pPanel->GetSelection(tMainSelection);
	tMainSelection.SetSelectPath(_pPanel->GetReader()->GetPath());
	tMainSelection.SetReader(_pPanel->GetReader());
	
	if (tMainSelection.GetSize() == 0)
		g_tMainFrame.GetClip()->Set(CLIP_NONE);
	else
		g_tMainFrame.GetClip()->Set(CLIP_CUT);

	g_Log.Write("CutClip [%s]", _pPanel->GetReader()->GetPath().c_str());
}

void CmdPanelImp::PasteClip()
{
	ClipState state = g_tMainFrame.GetClip()->GetState();

	if (state == CLIP_NONE) return;

	if (state == CLIP_COPY) 	
	{
		CopyPaste();
		Refresh();
	}
	else if (state == CLIP_CUT) 
	{
		CutPaste();
		Refresh();
	}
}

void	CmdPanelImp::CopyPaste()
{
	Selection& _tMainSelection = *(g_tMainFrame.GetSelection());

	if (_tMainSelection.GetSize() <= 0) return;

	Selection	tSelectionCopy;

	g_Log.Write("PasteClip Copy Start 1 - [%d] [%s] [%s] !!!", 
						_tMainSelection.GetSize(),
						_tMainSelection[0]->sType.c_str(), 
						_pPanel->GetReader()->GetInitType().c_str());

	// Source 나 Target이 file이면 그대로 이용한다.
	if (_tMainSelection[0]->sType == "file://" && 
		_pPanel->GetReader()->GetInitType() == "file://")
	{
		_tMainSelection.ExpandDir(_tMainSelection.GetReader(), false);

		// /tmp 에 복사하고 나서 옮긴다.
		g_Log.Write("PasteClip Copy Start !!!");
		_pPanel->GetReader()->Copy(_tMainSelection);
		g_Log.Write("PasteClip Copy End !!!");
	}
	else
	{
		bool	bUseGlobalReaderCtl = false;

		// 패널이 두개이기 때문에 mainframe 의 panel에서 Type에 맞는 데이터를 읽어 온다.
		Reader* pReader = g_tMainFrame.GetPanel(0)->GetReader(_tMainSelection[0]->sType);
		if (pReader == NULL || pReader->GetInitType() != _tMainSelection[0]->sType)
		{
			pReader = g_tMainFrame.GetPanel(1)->GetReader(_tMainSelection[0]->sType);
			
			if (pReader == NULL || pReader->GetInitType() != _tMainSelection[0]->sType)
			{
				// 그래도 없으면 만들서 넣어야 함.
				string::size_type p = _tMainSelection[0]->sType.find("://");
				string sInitName;

				if (p == string::npos)
					sInitName = _tMainSelection[0]->sType;
				else
					sInitName = _tMainSelection[0]->sType.substr(0, p);
		
				// 이때는 g_tReaderCtl로 읽어 들여옴.
				pReader = g_tReaderCtl.Get(sInitName);
				if (pReader == NULL)
				{
					MsgBox(gettext("Error"), gettext("plugin not found."));
					return;
				}

				g_Log.Write("g_tReaderCtl USE [%p] [%s]", pReader, sInitName.c_str());

				bUseGlobalReaderCtl = true;	
				if (p == string::npos)
					sInitName = _tMainSelection[0]->sType;
				else
					sInitName = _tMainSelection[0]->sType.substr(p+3);

				g_Log.Write("g_tReaderCtl USE [%p] [%s]", pReader, sInitName.c_str());

				g_Log.Write("%s", pReader->GetReaderName().c_str());
				pReader->Init(sInitName);
				
				if (pReader->GetConnected() == false) 
				{
					pReader->Destroy();
				}
				g_Log.Write("g_tReaderCtl Init [%p] [%s]", pReader, sInitName.c_str());
			}
		}

		if (pReader->GetConnected() == false)
		{
			pReader->Init(_tMainSelection[0]->sType);
			if (pReader->GetConnected() == false) return;
		}

		_tMainSelection.ExpandDir(pReader, false);

		// /tmp 에 복사하고 나서 옮긴다.
		if (pReader->Copy(_tMainSelection, "/tmp/", &tSelectionCopy))
		{
			g_Log.Write("File Copy Complite !!!");
		
			if (_pPanel->GetReader()->GetInitType() == "file://")
			{
				g_Log.Write("File Copy Move !!!");

				// 파일일 경우는 이동한다.
				_pPanel->GetReader()->Move(tSelectionCopy);
			}
			else
			{	
				g_Log.Write("File Copy /tmp Remove !!!");

				// 파일이 아닌 경우는 /tmp에서 다른 곳으로 복사 한 후
				// /tmp에 있는 내용은 지운다.
				if (_pPanel->GetReader()->Paste(tSelectionCopy))
					if ((pReader=_pPanel->GetReader("file://")) != NULL)
						pReader->Remove(tSelectionCopy, false);
			}
		}

		if (bUseGlobalReaderCtl == true)
			pReader->Destroy();
	}
}

void 	CmdPanelImp::CutPaste()
{
	Selection& _tMainSelection = *(g_tMainFrame.GetSelection());

	if (_tMainSelection.GetSize() <= 0) return;
	
	g_Log.Write("PasteClip CLIP_CUT Start - [%d] [%s] [%s] !!!", 
						_tMainSelection.GetSize(),
						_tMainSelection[0]->sType.c_str(),
						_pPanel->GetReader()->GetInitType().c_str());

	if (_tMainSelection[0]->sType == "file://" && _pPanel->GetReader()->GetInitType() == "file://")
	{
		_tMainSelection.ExpandDir(_tMainSelection.GetReader(), false);
		// sType이 file 이면 그냥 옮긴다.
		_pPanel->GetReader()->Move(_tMainSelection);
	}
	else
	{
		bool	bUseGlobalReaderCtl = false;

		// 패널이 두개이기 때문에 mainframe 의 panel에서 Type에 맞는 데이터를 읽어 온다.
		Reader* pReader = g_tMainFrame.GetPanel(0)->GetReader(_tMainSelection[0]->sType);
		if (pReader == NULL || pReader->GetInitType() != _tMainSelection[0]->sType)
		{
			pReader = g_tMainFrame.GetPanel(1)->GetReader(_tMainSelection[0]->sType);
			if (pReader == NULL || pReader->GetInitType() != _tMainSelection[0]->sType)
			{
				// 그래도 없으면 만들서 넣어야 함.
				string::size_type p = _tMainSelection[0]->sType.find("://");
				string sInitName;

				if (p != string::npos)
					sInitName = _tMainSelection[0]->sType;
				else
					sInitName = _tMainSelection[0]->sType.substr(0, p);

				// 이때는 g_tReaderCtl로 읽어 들여옴.
				pReader = g_tReaderCtl.Get(sInitName);
				if (pReader == NULL)
				{
					MsgBox(gettext("Error"), gettext("plugin not found."));
					return;
				}

				bUseGlobalReaderCtl = true;
				if (p == string::npos)
					sInitName = _tMainSelection[0]->sType;
				else
					sInitName = _tMainSelection[0]->sType.substr(p+3);

				pReader->Init(sInitName);
				if (pReader->GetConnected() == false) return;
			}
		}

		if (pReader->GetConnected() == false)
		{
			pReader->Init(_tMainSelection[0]->sType);
			if (pReader->GetConnected() == false) return;
		}

		_tMainSelection.ExpandDir(pReader, false);

		Selection	tSelectionMove;

		// /tmp 에 복사하고 나서 옮긴다.
		if (pReader->Copy(_tMainSelection, "/tmp/", &tSelectionMove))
		{
			Reader*		pCurReader = _pPanel->GetReader();
			if (pCurReader->GetInitType() == "file://")
			{
				// 파일일 경우는 이동한다.
				if (pCurReader->Move(tSelectionMove))
					pCurReader->Remove(_tMainSelection, false); // 이동하고 원본 삭제
			}
			else
			{	
				// 파일이 아닌 경우는 /tmp에서 다른 곳으로 복사 한 후
				// /tmp에 있는 내용은 지운다.
				if (pCurReader->Paste(tSelectionMove))
					if ((pReader = _pPanel->GetReader("file://")) != NULL)
					{
						pReader->Remove(tSelectionMove, false);	// Tmp 디렉토리 지움.
						pCurReader->Remove(_tMainSelection, false); // 원본 삭제
					}
			}
		}

		if (bUseGlobalReaderCtl == true)
			pReader->Destroy();
	}
}

void	CmdPanelImp::Key_ESC()
{
	if (g_tMainFrame.GetScreenSync())
	{
		g_tMainFrame.SetScreenSync(false);
		return;
	}

	if (_pPanel->GetReader()->GetReaderName() != "file")
	{
		_pPanel->PluginClose("file");
	}
	else
	{
		CursesDestroy();
		cbreak();
		noecho();
		getch();
		CursesInit();
	}
	g_tMainFrame.Refresh(false);
}

void	CmdPanelImp::Shell()
{
	if (_pPanel->GetReader()->GetInitType() == "file://")
		g_tMainFrame.CmdShell(true);
	else
		MsgBox("Error", "shell is local file system use only.");
}

void	CmdPanelImp::Rename()
{
	_pPanel->GetReader()->Rename(_pPanel->GetCurFile());
	Refresh();
}

void	CmdPanelImp::Mkdir()
{
	_pPanel->GetReader()->Mkdir(); 
	Refresh();
}

void CmdPanelImp::GoRoot()
{ 
	_pPanel->Read("/");
	if (g_tMainFrame.GetScreenSync()) ViewSync();
	g_tMainFrame.Refresh(false);;
}

void CmdPanelImp::GoHome() 
{ 
	_pPanel->Read("~");
	if (g_tMainFrame.GetScreenSync()) ViewSync();
	g_tMainFrame.Refresh(false);;
}

void CmdPanelImp::GoParent()
{ 
	_pPanel->Read("..");
	if (g_tMainFrame.GetScreenSync()) ViewSync();
	g_tMainFrame.Refresh(false);;
}

void CmdPanelImp::BoxCodeChange()
{
	g_tMainFrame.LineCodeChange();
}

void CmdPanelImp::HiddenFileView()
{
	g_Log.Write("_pPanel->GetHiddenView() [%s]", _pPanel->GetHiddenView() ? "TRUE" : "FALSE");
	_pPanel->SetHiddenView();
	Refresh();
}

// sort
void    CmdPanelImp::SortChange()
{
	_pPanel->_nSort++;
	if (_pPanel->_nSort == 5) _pPanel->_nSort=0;
	switch(_pPanel->_nSort)
	{
		case 0: SortName();    break;
		case 1: SortExt();    break;
		case 2: SortSize();    break;
		case 3: SortTime(); break;
		case 4: SortColor(); break;
	}
}

void CmdPanelImp::SortName()
{
	_pPanel->SetSortMethod(SORT_NAME);
	_pPanel->Sort();
}

void CmdPanelImp::SortExt()
{
	_pPanel->SetSortMethod(SORT_EXT);
	_pPanel->Sort();
}

void CmdPanelImp::SortSize()
{
	_pPanel->SetSortMethod(SORT_SIZE);
	_pPanel->Sort();
}

void CmdPanelImp::SortTime()
{
	_pPanel->SetSortMethod(SORT_TIME);
	_pPanel->Sort();
}

void CmdPanelImp::SortColor()
{
	_pPanel->SetSortMethod(SORT_COLOR);
	_pPanel->Sort();
}

void CmdPanelImp::ColumnAuto() 
{ 
	_pPanel->SetViewColumn(0);
}

void CmdPanelImp::Column1()
{ 
	_pPanel->SetViewColumn(1);
}

void CmdPanelImp::Column2()
{ 
	_pPanel->SetViewColumn(2);
}

void CmdPanelImp::Column3()
{ 
	_pPanel->SetViewColumn(3);
}

void CmdPanelImp::Column4()
{ 
	_pPanel->SetViewColumn(4);
}

void CmdPanelImp::DefaultCfgFileChg()
{
	File	tFile;
	tFile.sFullName = g_tCfg.GetValue("CfgHome") + "default.cfg";
	EditorChoice(false, "", &tFile);

	g_tCfg.Load((g_tCfg.GetValue("CfgHome") + "default.cfg").c_str());
	g_tColorCfg.Load((g_tCfg.GetValue("CfgHome") + "colorset.cfg").c_str());
	g_tKeyCfg.Load((g_tCfg.GetValue("CfgHome") + "keyset.cfg").c_str());
}

void CmdPanelImp::ColorCfgFileChg()
{
	File	tFile;
	tFile.sFullName = g_tCfg.GetValue("CfgHome") + "colorset.cfg";
	EditorChoice(false, "", &tFile);
	g_tCfg.Load((g_tCfg.GetValue("CfgHome") + "default.cfg").c_str());
	g_tColorCfg.Load((g_tCfg.GetValue("CfgHome") + "colorset.cfg").c_str());
	g_tKeyCfg.Load((g_tCfg.GetValue("CfgHome") + "keyset.cfg").c_str());
}

void CmdPanelImp::KeyCfgFileChg()
{
	File	tFile;
	tFile.sFullName = g_tCfg.GetValue("CfgHome") + "keyset.cfg";
	EditorChoice(false, "", &tFile);
	g_tCfg.Load((g_tCfg.GetValue("CfgHome") + "default.cfg").c_str());
	g_tColorCfg.Load((g_tCfg.GetValue("CfgHome") + "colorset.cfg").c_str());
	g_tKeyCfg.Load((g_tCfg.GetValue("CfgHome") + "keyset.cfg").c_str());
}

void CmdPanelImp::Copy()
{
	g_tMainFrame.Copy();
}

void CmdPanelImp::Move()
{
	g_tMainFrame.Move();
}

void CmdPanelImp::MountList()
{
	_pPanel->MountList();
}

void CmdPanelImp::SizeInfo()
{
	Selection tSelection;
	_pPanel->GetSelection(tSelection);

	void*  pWin = MsgWaitBox( 	gettext("Wait"),
								gettext("Please wait !!! - Cancel Key [ESC]"));

	tSelection.ExpandDir(_pPanel->GetReader(), true);

	MsgWaitEnd(pWin);

	String sMsg;
	sMsg.Append(gettext("  Selected files     %10s  /  %10s byte  "), 
				toregular(tSelection.GetSize()).c_str(),
				toregular(tSelection.CalcSize()).c_str());

	MsgBox(	gettext("Size info"), sMsg.c_str());
}
